@@ -1,24 +1,18 @@
-# Copyright (c) 2016 The OpenTracing Authors.
+# Copyright The OpenTracing Authors
+# Copyright Uber Technologies, Inc
 #
-# Copyright (c) 2015 Uber Technologies, Inc.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
+# http://www.apache.org/licenses/LICENSE-2.0
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import absolute_import
 
 # The following tags are described in greater detail at the following url:
@@ -41,6 +35,12 @@ SPAN_KIND_RPC_CLIENT = 'client'
 # Marks a span representing the server-side of an RPC or other remote call
 SPAN_KIND_RPC_SERVER = 'server'
 
+# Marks a span representing the consumer-side of a messaging call
+SPAN_KIND_CONSUMER = 'consumer'
+
+# Marks a span representing the producer-side of a messaging call
+SPAN_KIND_PRODUCER = 'producer'
+
 # ---------------------------------------------------------------------------
 # ERROR indicates whether a Span ended in an error state.
 # ---------------------------------------------------------------------------
